;===============================================================================
;
; Andrew Jacobs
; 2018-10-11
;-------------------------------------------------------------------------------

		.65816 		; But actually 6502 code

;===============================================================================
; Workspace
;-------------------------------------------------------------------------------

		.page0
		.org	$10
		
CH		.space	1		;
BITS		.space	1

LEN		.space	1		; Length of input data
PTR		.space	2		; Address of input data

;===============================================================================
; The emulator expects the S28 to contain a ROM image so this is kicked off via
; the dummy reset vector.
;-------------------------------------------------------------------------------

		.code
		.org	$f000
		
Reset:
		sei
		cld
		ldx	#$ff
		txs
		
		lda	#TEST_END-TEST
		sta	LEN
		lda	#<TEST
		sta	PTR+0
		lda	#>TEST
		sta	PTR+1
		
		jsr	Base64
		
AllDone:
		wdm	#$ff		; Stop the emulator
		bra	AllDone	

; This should encode as:
;
; QSBmb29sIHRoaW5rcyBoaW1zZWxmIHRvIGJlIHdpc2UsIGJ1dCBhIHdpc2UgbWFuIGtub3dzIGhp
; bXNlbGYgdG8gYmUgYSBmb29sLg==

TEST:		.byte 	"A fool thinks himself to be wise, but a wise man "
		.byte 	"knows himself to be a fool."
TEST_END:

; Output Utilities

UartTx:
		wdm	#$01		; Print a character
		rts

NewLine:
		lda	#10		; LF
		bra	UartTx

;-------------------------------------------------------------------------------

Base64:
		ldy	#0		; Load index into data
		sty	CH		; .. and initialise 
		sty	BITS
		repeat
		 jsr	Extract		; Output 
		 jsr	UartTx
		 jsr	Extract
		 jsr	UartTx
		 jsr	ExtractPad
		 jsr	UartTx
		 jsr	ExtractPad
		 jsr	UartTx
		 cpy	LEN		; Until out of data
		until cs
		jsr	NewLine
		rts
		
		
ExtractPad:
		cpy	LEN
		if	cs
		 lda	BITS
		 beq	.Done
		 bpl	Extract
.Done		 lda	#'='
		 rts
		endif
Extract:
		lda	#0		; Shift out six bits
		jsr	Shift2
		jsr 	Shift2
		jsr	Shift2
		tax			; And translate to character
		lda	LEXICON,x
		rts
		 		
Shift2:
		ldx	BITS
		if eq
		 cpy	LEN
		 if cc
		  pha
		  lda	(PTR),y
		  sta	CH
		  iny
		  lda	#8
		  sta	BITS
		  pla
		 endif
		endif
		asl	CH
		rol	a
		asl	CH
		rol	a
		dec	BITS
		dec	BITS
		rts
		
LEXICON:	.byte	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		.byte	"abcdefghijklmnopqrstuvwxyz"
		.byte	"0123456789+/"
		
	
		.org	$fffc
		.word	Reset		; Dummy RESET vector
		
		.end		